@@ -434,10 +434,9 @@ impl DPEventLoop {
 
   fn handle_writer_acknack_action(&mut self, _event: &Event) {
     while let Ok((acknack_sender_prefix, acknack_message)) = self.ack_nack_reciever.try_recv() {
-      let target_writer_entity_id = { acknack_message.writer_id };
       let writer_guid = GUID::new_with_prefix_and_id(
         self.domain_info.domain_participant_guid.guidPrefix,
-        target_writer_entity_id,
+        acknack_message.writer_id,
       );
       if let Some(found_writer) = self.writers.get_mut(&writer_guid.entityId) {
         if found_writer.is_reliable() {
@@ -575,6 +574,7 @@ impl DPEventLoop {
   } // fn 
 
   fn remote_participant_lost(&mut self, participant_guid_prefix: GuidPrefix ) {
+    info!("remote_participant_lost guidPrefix={:?}", &participant_guid_prefix );
     // Discovery has already removed Particiapnt from Discovery DB
     // Now we have to remove any ReaderProxies and WriterProxies belonging
     // to that particiapnt, so that we do not send messages to them anymore.
@@ -691,7 +691,7 @@ mod tests {
       mio_channel::channel();
 
     let ddshc = Arc::new(RwLock::new(DDSCache::new()));
-    let discovery_db = Arc::new(RwLock::new(DiscoveryDB::new()));
+    let discovery_db = Arc::new(RwLock::new(DiscoveryDB::new(GUID::new_particiapnt_guid())));
 
     let domain_info = DomainInfo {
       domain_participant_guid: GUID::default(),
