@@ -27,7 +27,7 @@ use mio_extras::channel as mio_channel;
 use log::{debug, info, warn, trace, error};
 use std::fmt;
 
-use std::collections::{HashSet, BTreeMap, HashMap, };
+use std::collections::{BTreeMap};
 use std::time::Duration as StdDuration;
 use enumflags2::BitFlags;
 
@@ -59,7 +59,8 @@ pub(crate) struct Reader {
   status_sender: mio_channel::SyncSender<DataReaderStatus>,
 
   dds_cache: Arc<RwLock<DDSCache>>,
-  seqnum_instant_map: HashMap<SequenceNumber, Timestamp>,
+  #[cfg(test)]
+  seqnum_instant_map: BTreeMap<SequenceNumber, Timestamp>,
   topic_name: String,
   qos_policy: QosPolicies,
 
@@ -99,7 +100,8 @@ impl Reader {
       topic_name,
       qos_policy,
 
-      seqnum_instant_map: HashMap::new(),
+      #[cfg(test)]
+      seqnum_instant_map: BTreeMap::new(),
       my_guid: guid ,
       enpoint_attributes: EndpointAttributes::default(),
 
@@ -130,7 +132,7 @@ impl Reader {
     debug!("history cache !!!! {:?}", cc);
 
     match cc {
-      Some(cc) => cc.data_value.clone(), //  Some(DDSData::new(cc.data_value.as_ref().unwrap().clone())),
+      Some(cc) => Some(cc.data_value.clone()) , //  Some(DDSData::new(cc.data_value.as_ref().unwrap().clone())),
       None => None,
     }
   }
@@ -273,6 +275,7 @@ impl Reader {
   }
 
   // Used for test/debugging purposes
+  #[cfg(test)]
   pub fn get_history_cache_change(&self, sequence_number: SequenceNumber) -> Option<CacheChange> {
     debug!("{:?}", sequence_number);
     let dds_cache = self.dds_cache.read().unwrap();
@@ -302,13 +305,15 @@ impl Reader {
     match offered_qos.compliance_failure_wrt( &self.qos_policy ) {
       None => { // success, update or insert
         let count_change =
-          self.matched_writer_update(proxy);
+          self.matched_writer_update(proxy.clone()); // need clone here to log depending on this
         if count_change > 0 {
           self.writer_match_count_total += count_change;
           self.send_status_change(DataReaderStatus::SubscriptionMatched{
               total: CountWithChange::new(self.writer_match_count_total, count_change ), 
               current: CountWithChange::new(self.matched_writers.len() as i32, count_change ),
-          })
+          });
+          info!("Matched new remote writer on topic={:?} writer= {:?}", 
+                self.topic_name, &proxy);
         }
       }
       Some(bad_policy_id) => { // no QoS match
@@ -445,11 +450,14 @@ impl Reader {
 
     self.make_cache_change(data, data_flags, instant, writer_guid, no_writers);
     // Add to own track-keeping datastructure
+    #[cfg(test)]
     self.seqnum_instant_map.insert(seq_num, instant);
 
     self.notify_cache_change();
   }
 
+  // Returns if responding with ACKNACK?
+  // TODO: Return value seems to go unused in callers.
   pub fn handle_heartbeat_msg(
     &mut self,
     heartbeat: Heartbeat,
@@ -461,19 +469,29 @@ impl Reader {
 
     // Added in order to test stateless actions. TODO
     if !self.matched_writers.contains_key(&writer_guid) {
-      return false;
+      info!("HEARTBEAT for {:?}, but no writer proxy available. topic={:?}", writer_guid, self.topic_name);
+      return false
+    }
+    // sanity check
+    if heartbeat.first_sn < SequenceNumber::default() {
+      warn!("Writer {:?} advertised SequenceNumbers from {:?} to {:?}!",
+          writer_guid, heartbeat.first_sn, heartbeat.last_sn);
     }
 
     let writer_proxy = match self.matched_writer_lookup(writer_guid) {
       Some(wp) => wp,
-      None => return false, // Matching writer not found
+      None => {
+        error!("Writer proxy disappeared 1!");
+        return false
+      } // Matching writer not found
     };
 
     let mut mr_state = mr_state;
     mr_state.unicast_reply_locator_list = writer_proxy.unicast_locator_list.clone();
 
     if heartbeat.count <= writer_proxy.received_heartbeat_count {
-      return false;
+      // This heartbeat was already seen an processed.
+      return false
     }
     writer_proxy.received_heartbeat_count = heartbeat.count;
 
@@ -481,39 +499,64 @@ impl Reader {
     let removed_instances = writer_proxy.irrelevant_changes_up_to(heartbeat.first_sn);
 
     // Remove instances from DDSHistoryCache
-    let mut cache = match self.dds_cache.write() {
-      Ok(rwlock) => rwlock,
-      // TODO: Should we panic here? Are we allowed to continue with poisoned DDSCache?
-      Err(e) => panic!("The DDSCache of is poisoned. Error: {}", e),
-    };
-    for instant in removed_instances.iter() {
-      match cache.from_topic_remove_change(&self.topic_name, instant) {
-        Some(_) => (),
-        None => warn!("WriterProxy told to remove an instant which was not present"),
+    {
+      // Create local scope so that dds_cache write lock is dropped ASAP
+      let mut cache = match self.dds_cache.write() {
+        Ok(rwlock) => rwlock,
+        // TODO: Should we panic here? Are we allowed to continue with poisoned DDSCache?
+        Err(e) => panic!("The DDSCache of is poisoned. Error: {}", e),
+      };
+      for instant in removed_instances.values() {
+        match cache.from_topic_remove_change(&self.topic_name, instant) {
+          Some(_) => (),
+          None => warn!("WriterProxy told to remove an instant which was not present"),
+        }
       }
     }
-    drop(cache);
 
+    // this is duplicate code from above, but needed, because we need another mutable borrow.
+    // TODO: Maybe could be written in some sensible way.
     let writer_proxy = match self.matched_writer_lookup(writer_guid) {
       Some(wp) => wp,
-      None => return false, // Matching writer not found
+      None => {
+        error!("Writer proxy disappeared 2!");
+        return false
+      } // Matching writer not found
     };
 
-    // See if ACKNACK is needed.
-    // TODO: too convoluted and inefficient block
-    if writer_proxy.changes_are_missing(heartbeat.first_sn, heartbeat.last_sn) || !final_flag_set {
-      let missing_seqnums =
+    // See if ACKNACK is needed, and generate one.
+    let missing_seqnums =
         writer_proxy.get_missing_sequence_numbers(heartbeat.first_sn, heartbeat.last_sn);
-      let seqnum_base_and_top =
-        match (missing_seqnums.iter().min(), missing_seqnums.iter().max()) {
-          (Some(&base),Some(&top)) => (base,top),
-          (_,_) => (heartbeat.last_sn, heartbeat.last_sn), // this should not happen because of "if" above
+    
+    // Interpretation of final flag in RTPS spec 
+    // 8.4.2.3.1 Readers must respond eventually after receiving a HEARTBEAT with final flag not set
+    // 
+    // Upon receiving a HEARTBEAT Message with final flag not set, the Reader must respond 
+    // with an ACKNACK Message. The ACKNACK Message may acknowledge having received all 
+    // the data samples or may indicate that some data samples are missing.
+    // The response may be delayed to avoid message storms.
+
+    if ! missing_seqnums.is_empty() || ! final_flag_set {
+      // report of what we have.
+      // We claim to have received all SNs before "base" and produce a set of missing 
+      // sequence numbers that are >= base.
+      let reader_sn_state =
+        match missing_seqnums.iter().next()  {
+          Some(&first_missing) => {
+              // Here we assume missing_seqnums are returned in order.
+              // Limit the set to maximum that can be sent in acknack submessage..
+            SequenceNumberSet
+              ::from_base_and_set(first_missing, 
+                &BTreeSet::from_iter(missing_seqnums.iter()
+                                      .map(|s| *s)
+                                      .take_while( |sn| sn < &(first_missing + SequenceNumber::from(256)) ))
+                )
+            }
+
+          // Nothing missing. Report that we have all we have.
+          None => SequenceNumberSet::new_empty(writer_proxy.all_ackable_before()),           
         };
 
-      let reader_sn_state = SequenceNumberSet::from_base_and_set(
-        seqnum_base_and_top.0 , &BTreeSet::from_iter(missing_seqnums) 
-        //TODO: should not be needed
-        );
 
       let response_ack_nack = AckNack {
         reader_id: self.get_entity_id(),
@@ -523,12 +566,21 @@ impl Reader {
       };
 
       self.sent_ack_nack_count += 1;
+
+      // Sanity check
+      if response_ack_nack.reader_sn_state.base() > heartbeat.last_sn + SequenceNumber::new(1) {
+        error!("OOPS! AckNack sanity check tripped: HEARTBEAT = {:?} ACKNACK = {:?}",
+          &heartbeat, &response_ack_nack
+          );
+      }
+
+
       // The acknack can be sent now or later. The rest of the RTPS message
       // needs to be constructed. p. 48
-
       self.send_acknack(response_ack_nack, mr_state);
-      return true;
+      return true
     }
+    
     false
   } // fn 
 
@@ -562,34 +614,31 @@ impl Reader {
     // } else {
     //   return;
     // };
+
+
+
     // Irrelevant sequence numbers communicated in the Gap message are
     // composed of two groups:
-    let mut irrelevant_changes_set = HashSet::new();
-
     //   1. All sequence numbers in the range gapStart <= sequence_number < gapList.base
-    for seq_num_i64 in i64::from(gap.gap_start)..i64::from(gap.gap_list.base()) {
-      //TODO: Can we do iteration range directly on SequenceNumber type?
-      irrelevant_changes_set.insert(SequenceNumber::from(seq_num_i64));
-    }
+    let mut removed_changes : BTreeSet<Timestamp> = 
+      writer_proxy.irrelevant_changes_range(gap.gap_start, gap.gap_list.base())
+        .values()
+        .map(|s|*s)
+        .collect();
+
     //   2. All the sequence numbers that appear explicitly listed in the gapList.
     for seq_num in gap.gap_list.iter() {
-      irrelevant_changes_set.insert(seq_num);
+      writer_proxy.set_irrelevant_change(seq_num)
+        .map( |t| removed_changes.insert(t) );
     }
 
-    // Remove from writerProxy and DDSHistoryCache
-    let mut removed_instances = Vec::new();
-    for seq_num in &irrelevant_changes_set {
-      match writer_proxy.set_irrelevant_change(*seq_num) {
-        Some(i) => removed_instances.push(i),
-        None => (),
-      };
-    }
+    // Remove from DDSHistoryCache
     let mut cache = match self.dds_cache.write() {
       Ok(rwlock) => rwlock,
       // TODO: Should we panic here? Are we allowed to continue with poisoned DDSCache?
       Err(e) => panic!("The DDSCache of is poisoned. Error: {}", e),
     };
-    for instant in &removed_instances {
+    for instant in &removed_changes {
       cache.from_topic_remove_change(&self.topic_name, instant);
     }
 
