@@ -204,8 +204,8 @@ impl Writer {
       cahce_cleaning_perioid: Duration::from_secs(2 * 60),
       nack_respose_delay: Duration::from_millis(200),
       nack_suppression_duration: Duration::from_millis(0),
-      last_change_sequence_number: SequenceNumber::from(0),
-      first_change_sequence_number: SequenceNumber::from(0),
+      first_change_sequence_number: SequenceNumber::from(1), // first = 1, last = 0
+      last_change_sequence_number: SequenceNumber::from(0),  // means we have nothing to write
       data_max_size_serialized: 999999999,
       my_guid: guid,
       //enpoint_attributes: EndpointAttributes::default(),
@@ -463,7 +463,8 @@ impl Writer {
       let hb_message = MessageBuilder::new()
         .ts_msg(self.endianness, Some(Timestamp::now()) )
         .heartbeat_msg(self, EntityId::ENTITYID_UNKNOWN, final_flag, liveliness_flag)
-        .add_header_and_build(self.my_guid.guidPrefix);      
+        .add_header_and_build(self.my_guid.guidPrefix);
+      debug!("Writer {:?} topic={:} HEARTBEAT {:?}", self.get_guid().entityId, self.topic_name(), hb_message );
       self.send_message_to_readers(DeliveryMode::Multicast, &hb_message, 
                                     &mut self.readers.values())
     }
@@ -498,6 +499,15 @@ impl Writer {
     // Update the ReaderProxy
     let last_seq = self.last_change_sequence_number; // to avoid borrow problems
 
+    // sanity check requested sequence numbers
+    match an.reader_sn_state.iter().next().map( i64::from ) {
+      None => (), // ok
+      Some(0) => 
+        warn!("Request for SN zero! : {:?}",an),
+      Some(_) => (), // ok
+    }
+    let my_topic = self.my_topic_name.clone(); // for debugging
+
     self.update_ack_waiters( 
       GUID::new(reader_guid_prefix, an.reader_id) , 
       Some(an.reader_sn_state.base()));
@@ -517,14 +527,12 @@ impl Writer {
         }
         // Sanity Check
         if an.reader_sn_state.base() > last_seq + SequenceNumber::from(1) { // more sanity
-          warn!("ACKNACK from {:?} acks up to before {:?} but I have only up to {:?}",
-            reader_proxy.remote_reader_guid, reader_proxy.unsent_changes, last_seq);
+          warn!("ACKNACK from {:?} acks {:?}, but I have only up to {:?} count={:?} topic={:?}",
+            reader_proxy.remote_reader_guid, an.reader_sn_state, last_seq, an.count, my_topic);
         }
-        // TODO: The following check is rather expensive. Maybe should turn it off
-        // in release build?
-        if let Some( max_req_sn ) = an.reader_sn_state.iter().max() { // sanity check
+        if let Some( max_req_sn ) = an.reader_sn_state.iter().next_back() { // sanity check
           if max_req_sn > last_seq {
-            warn!("ACKNACK from {:?} requests {:?} but I have only up to {:?}",
+            warn!("ACKNACK from {:?} requests {:?} but I have only up to {:?}",
               reader_proxy.remote_reader_guid, 
               an.reader_sn_state.iter().collect::<Vec<SequenceNumber>>(), last_seq);
           }
@@ -567,70 +575,87 @@ impl Writer {
   }
 
   // Send out missing data
+
   fn handle_repair_data_send(&mut self, to_reader: GUID) {
-    if let Some(mut reader_proxy) = self.matched_reader_remove(to_reader) {
-      let reader_guid = reader_proxy.remote_reader_guid;
-      let mut partial_message = MessageBuilder::new()
-        .dst_submessage(self.endianness, reader_guid.guidPrefix)
-        .ts_msg(self.endianness, Some(Timestamp::now())); 
-        // TODO: This timestamp should probably not be
-        // the current (retransmit) time, but the initial sample production timestamp,
-        // i.e. should be read from DDSCache (and be implemented there)
-      debug!("Repair data send due to ACKNACK. ReaderProxy Unsent changes: {:?}",
-              reader_proxy.unsent_changes);
-
-      let mut no_longer_relevant = Vec::new();
-      let mut found_data = false;
-      if let Some(&unsent_sn) = reader_proxy.unsent_changes.iter().next() {
-        match self.sequence_number_to_instant(unsent_sn) {
-          Some(timestamp) => {
-            if let Some(cache_change) = self.dds_cache.read().unwrap()
-                .from_topic_get_change(&self.my_topic_name, &timestamp) {
-              partial_message = partial_message
-                  .data_msg(cache_change.clone(), // TODO: We should not clone, too much copying
-                            reader_guid.entityId, // reader
-                            self.my_guid.entityId, // writer
-                            self.endianness); 
-              // TODO: Here we are cloning the entire payload. We need to rewrite the transmit path to avoid copying.
-            } else {
-              no_longer_relevant.push(unsent_sn);
-            }
-          }
-          None => {
-            error!("handle ack_nack writer {:?} seq.number {:?} missing from instant map", 
-                    self.my_guid, unsent_sn);
-            no_longer_relevant.push(unsent_sn);
-          }  
-        } // match
-        reader_proxy.unsent_changes.remove(&unsent_sn);
-        found_data = true;
-      }
-      // Add GAP submessage, if some chache changes could not be found.
-      if ! no_longer_relevant.is_empty() {
-        partial_message = partial_message.gap_msg(BTreeSet::from_iter(no_longer_relevant), &self, reader_guid);
-      }
-      let data_gap_msg = partial_message
-        .add_header_and_build(self.my_guid.guidPrefix);
-
-      self.send_message_to_readers(DeliveryMode::Unicast, &data_gap_msg,
-                &mut std::iter::once(&reader_proxy));
-
-      if found_data { // prime repair timer again, if data was found
-        // Try to send repait messages at 5x rate compared to usual deadline rate
-        let delay_to_next_message = 
-          self.qos_policies.deadline().map( |dl| dl.0 )
-            .unwrap_or(Duration::from_millis(100)) / 5;
-        self.timed_event_handler.as_mut().unwrap().set_timeout(
-          &chronoDuration::from(delay_to_next_message),
-          TimerMessageType::WriterSendRepairData{ to_reader: reader_proxy.remote_reader_guid },
-        );        
-      } else {
-        reader_proxy.repair_mode = false; // resume normal data sending
-      }
-
+    // Note: here we remove the reader from our reader map temporarily.
+    // Then we can mutate both the reader and other fields in self.
+    // Doing a .get_mut() on the reader map would make self immutable.
+    if let Some(reader_proxy) = self.readers.remove(&to_reader) {
+      // We use a worker function to ensure that afterwards we can insert the reader_proxy back.
+      // This technique ensures that all return paths lead to insertion.
+      let reader_proxy = self.handle_repair_data_send_worker(reader_proxy);
       // insert reader back
-      self.matched_reader_add(reader_proxy);      
-    }   
+      let reject = self.readers.insert(reader_proxy.remote_reader_guid, reader_proxy);
+      if reject.is_some() {
+        error!("Reader proxy was duplicated somehow??? {:?}",reject)
+      }
+    }
+  }
+
+  fn handle_repair_data_send_worker(&mut self, mut reader_proxy: RtpsReaderProxy) 
+    -> RtpsReaderProxy
+  {
+    // Note: The reader_proxy is now removed from readers map
+    let reader_guid = reader_proxy.remote_reader_guid;
+    let mut partial_message = MessageBuilder::new()
+      .dst_submessage(self.endianness, reader_guid.guidPrefix)
+      .ts_msg(self.endianness, Some(Timestamp::now())); 
+      // TODO: This timestamp should probably not be
+      // the current (retransmit) time, but the initial sample production timestamp,
+      // i.e. should be read from DDSCache (and be implemented there)
+    debug!("Repair data send due to ACKNACK. ReaderProxy Unsent changes: {:?}",
+            reader_proxy.unsent_changes);
+
+    let mut no_longer_relevant = Vec::new();
+    let mut found_data = false;
+    if let Some(&unsent_sn) = reader_proxy.unsent_changes.iter().next() {
+      match self.sequence_number_to_instant(unsent_sn) {
+        Some(timestamp) => {
+          if let Some(cache_change) = self.dds_cache.read().unwrap()
+              .from_topic_get_change(&self.my_topic_name, &timestamp) {
+            partial_message = partial_message
+                .data_msg(cache_change.clone(), // TODO: We should not clone, too much copying
+                          reader_guid.entityId, // reader
+                          self.my_guid.entityId, // writer
+                          self.endianness); 
+            // TODO: Here we are cloning the entire payload. We need to rewrite the transmit path to avoid copying.
+          } else {
+            no_longer_relevant.push(unsent_sn);
+          }
+        }
+        None => {
+          error!("handle ack_nack writer {:?} seq.number {:?} missing from instant map", 
+                  self.my_guid, unsent_sn);
+          no_longer_relevant.push(unsent_sn);
+        }  
+      } // match
+      reader_proxy.unsent_changes.remove(&unsent_sn);
+      found_data = true;
+    }
+    // Add GAP submessage, if some chache changes could not be found.
+    if ! no_longer_relevant.is_empty() {
+      partial_message = partial_message.gap_msg(BTreeSet::from_iter(no_longer_relevant), &self, reader_guid);
+    }
+    let data_gap_msg = partial_message
+      .add_header_and_build(self.my_guid.guidPrefix);
+
+    self.send_message_to_readers(DeliveryMode::Unicast, &data_gap_msg,
+              &mut std::iter::once(&reader_proxy));
+
+    if found_data { // prime repair timer again, if data was found
+      // Try to send repair messages at 5x rate compared to usual deadline rate
+      let delay_to_next_message = 
+        self.qos_policies.deadline().map( |dl| dl.0 )
+          .unwrap_or(Duration::from_millis(100)) / 5;
+      self.timed_event_handler.as_mut().unwrap().set_timeout(
+        &chronoDuration::from(delay_to_next_message),
+        TimerMessageType::WriterSendRepairData{ to_reader: reader_proxy.remote_reader_guid },
+      );        
+    } else {
+      reader_proxy.repair_mode = false; // resume normal data sending
+    }     
+
+    reader_proxy
   } // fn
 
   /// Removes permanently cacheChanges from DDSCache.
@@ -731,7 +756,7 @@ impl Writer {
       // matched QoS
       None => {
         let change =
-          self.matched_reader_update( reader_proxy );
+          self.matched_reader_update( reader_proxy.clone() );
         if change > 0 {
           self.matched_readers_count_total += change;
           self.send_status(DataWriterStatus::PublicationMatched { 
@@ -744,6 +769,9 @@ impl Writer {
               self.notify_new_data_to_all_readers(),
             _ => (),
           }
+          info!("Matched new remote reader on topic={:?} reader= {:?}", 
+                self.topic_name(), &reader_proxy);
+
         }
       }
       Some(bad_policy_id) => {
@@ -785,19 +813,26 @@ impl Writer {
   // internal helper. Same as above, but duplicates are an error.
   fn matched_reader_add(&mut self, reader_proxy: RtpsReaderProxy) {
     let guid = reader_proxy.remote_reader_guid;
-    if self.readers.insert(reader_proxy.remote_reader_guid, reader_proxy).is_some() {
+    if self.readers.insert(reader_proxy.remote_reader_guid, reader_proxy.clone()).is_some() {
       error!("Reader proxy with same GUID {:?} added already", guid);
+    } else {
+      info!("Added reader proxy. topic={:?} reader={:?}", self.topic_name(), reader_proxy );
     }
   }
 
   fn matched_reader_remove(&mut self, guid: GUID,) -> Option<RtpsReaderProxy> {
-    self.readers.remove(&guid)
+    let removed = self.readers.remove(&guid);
+    if removed.is_some() {
+      info!("Removed reader proxy. topic={:?} reader={:?}", self.topic_name(), removed );
+    }
+    removed
   }
 
 
   pub fn reader_lost(&mut self, guid: GUID)
   {
     if self.readers.contains_key(&guid) {
+      info!("reader_lost topic={:?} reader={:?}", self.topic_name(), &guid );
       self.matched_reader_remove(guid);
       //self.matched_readers_count_total -= 1; // this never decreases
       self.send_status(DataWriterStatus::PublicationMatched { 
