@@ -27,7 +27,10 @@ impl UDPSender {
   pub fn new(sender_port: u16) -> UDPSender {
     let saddr: SocketAddr = SocketAddr::new("0.0.0.0".parse().unwrap(), sender_port);
     let socket: UdpSocket = UdpSocket::bind(&saddr).unwrap();
-
+    // We set multicasting loop on so that we can hear other DomainParticipant
+    // instances running on the same host.
+    socket.set_multicast_loop_v4(true)
+      .unwrap_or_else(|e| { error!("Cannot set multicast loop on: {:?}",e); } );
     UDPSender { socket: socket }
   }
 
@@ -107,7 +110,7 @@ mod tests {
 
   #[test]
   fn udps_single_send() {
-    let listener = UDPListener::new(Token(0), "127.0.0.1", 10201);
+    let listener = UDPListener::new_unicast(Token(0), "127.0.0.1", 10201).unwrap();
     let sender = UDPSender::new(11201);
 
     let data: Vec<u8> = vec![0, 1, 2, 3, 4];
@@ -123,8 +126,8 @@ mod tests {
 
   #[test]
   fn udps_multi_send() {
-    let listener_1 = UDPListener::new(Token(0), "127.0.0.1", 10301);
-    let listener_2 = UDPListener::new(Token(1), "127.0.0.1", 10302);
+    let listener_1 = UDPListener::new_unicast(Token(0), "127.0.0.1", 10301).unwrap();
+    let listener_2 = UDPListener::new_unicast(Token(1), "127.0.0.1", 10302).unwrap();
     let sender = UDPSender::new(11301);
 
     let data: Vec<u8> = vec![5, 4, 3, 2, 1, 0];
